@@ -7,11 +7,14 @@ use std::io::ErrorKind;
 use std::path::Path;
 use std::process::ExitCode;
 
+use glob::Pattern;
 use lexopt::prelude::*;
 
+use threecpio::extract::{extract_cpio_archive, ExtractOptions};
+use threecpio::ranges::Ranges;
 use threecpio::{
-    create_cpio_archive, examine_cpio_content, extract_cpio_archive, list_cpio_content,
-    print_cpio_archive_count, LOG_LEVEL_DEBUG, LOG_LEVEL_INFO, LOG_LEVEL_WARNING,
+    create_cpio_archive, examine_cpio_content, list_cpio_content, print_cpio_archive_count,
+    LOG_LEVEL_DEBUG, LOG_LEVEL_INFO, LOG_LEVEL_WARNING,
 };
 
 #[derive(Debug)]
@@ -25,8 +28,24 @@ struct Args {
     list: bool,
     log_level: u32,
     archive: Option<String>,
+    make_directories: bool,
+    parts: Option<Ranges>,
+    patterns: Vec<Pattern>,
     preserve_permissions: bool,
     subdir: Option<String>,
+    to_stdout: bool,
+}
+
+impl Args {
+    fn extract_options(&self) -> ExtractOptions {
+        ExtractOptions::new(
+            self.make_directories,
+            self.parts.clone(),
+            self.patterns.clone(),
+            self.preserve_permissions,
+            self.subdir.clone(),
+        )
+    }
 }
 
 fn print_help() {
@@ -36,8 +55,8 @@ fn print_help() {
     {executable} --count ARCHIVE
     {executable} {{-c|--create}} [-v|--debug] [-C DIR] [ARCHIVE] < manifest
     {executable} {{-e|--examine}} ARCHIVE
-    {executable} {{-t|--list}} [-v|--debug] ARCHIVE
-    {executable} {{-x|--extract}} [-v|--debug] [-C DIR] [-p] [-s NAME] [--force] ARCHIVE
+    {executable} {{-t|--list}} [-v|--debug] [-P LIST] ARCHIVE [pattern...]
+    {executable} {{-x|--extract}} [-v|--debug] [-C DIR] [--make-directories] [-P LIST] [-p] [-s NAME] [--to-stdout] [--force] ARCHIVE [pattern...]
 
 Optional arguments:
   --count        Print the number of concatenated cpio archives.
@@ -46,11 +65,14 @@ Optional arguments:
   -t, --list     List the contents of the cpio archives.
   -x, --extract  Extract cpio archives.
   -C, --directory=DIR  Change directory before performing any operation.
+  --make-directories   Create leading directories where needed.
+  -P, --parts=LIST  Only operate on the cpio archives that matches LIST.
   -p, --preserve-permissions
                  Set permissions of extracted files to those recorded in the
                  archive (default for superuser).
   -s, --subdir   Extract the cpio archives into separate directories (using the
                  given name plus an incrementing number)
+  --to-stdout    Extract files to standard output
   -v, --verbose  Verbose output
   --debug        Debug output
   --force        Force overwriting existing files
@@ -71,12 +93,17 @@ fn parse_args() -> Result<Args, lexopt::Error> {
     let mut examine = 0;
     let mut extract = 0;
     let mut force = false;
+    let mut parts = None;
     let mut preserve_permissions = is_root();
     let mut list = 0;
     let mut log_level = LOG_LEVEL_WARNING;
     let mut directory = ".".into();
     let mut archive = None;
+    let mut make_directories = false;
+    let mut patterns = Vec::new();
     let mut subdir: Option<String> = None;
+    let mut to_stdout = false;
+    let mut arguments = Vec::new();
     let mut parser = lexopt::Parser::from_env();
     while let Some(arg) = parser.next()? {
         match arg {
@@ -102,6 +129,12 @@ fn parse_args() -> Result<Args, lexopt::Error> {
                 print_help();
                 std::process::exit(0);
             }
+            Long("make-directories") => {
+                make_directories = true;
+            }
+            Short('P') | Long("parts") => {
+                parts = Some(parser.value()?.parse()?);
+            }
             Short('p') | Long("preserve-permissions") => {
                 preserve_permissions = true;
             }
@@ -111,6 +144,9 @@ fn parse_args() -> Result<Args, lexopt::Error> {
             Short('t') | Long("list") => {
                 list = 1;
             }
+            Long("to-stdout") => {
+                to_stdout = true;
+            }
             Short('v') | Long("verbose") => {
                 if log_level <= LOG_LEVEL_INFO {
                     log_level = LOG_LEVEL_INFO;
@@ -126,6 +162,7 @@ fn parse_args() -> Result<Args, lexopt::Error> {
             Value(val) if archive.is_none() => {
                 archive = Some(val.string()?);
             }
+            Value(val) => arguments.push(val.string()?),
             _ => return Err(arg.unexpected()),
         }
     }
@@ -136,6 +173,17 @@ fn parse_args() -> Result<Args, lexopt::Error> {
         );
     }
 
+    if extract + list == 1 {
+        for argument in arguments {
+            let pattern = Pattern::new(&argument)
+                .map_err(|e| format!("invalid pattern '{argument}': {e}"))?;
+            patterns.push(pattern);
+        }
+    } else if !arguments.is_empty() {
+        let first = &arguments[0];
+        return Err(Value(first.into()).unexpected());
+    }
+
     if let Some(ref s) = subdir {
         if s.contains('/') {
             return Err(format!("Subdir '{s}' must not contain slashes!").into());
@@ -156,8 +204,12 @@ fn parse_args() -> Result<Args, lexopt::Error> {
         list: list == 1,
         log_level,
         archive,
+        make_directories,
+        parts,
+        patterns,
         preserve_permissions,
         subdir,
+        to_stdout,
     })
 }
 
@@ -259,7 +311,7 @@ fn main() -> ExitCode {
         }
     };
 
-    if args.extract {
+    if args.extract && !args.to_stdout {
         if let Err(e) = create_and_set_current_dir(&args.directory, args.force) {
             eprintln!("{executable}: Error: {e}");
             return ExitCode::FAILURE;
@@ -282,15 +334,21 @@ fn main() -> ExitCode {
             "extract content",
             extract_cpio_archive(
                 archive,
-                args.preserve_permissions,
-                args.subdir,
+                args.to_stdout.then_some(&mut stdout),
+                &args.extract_options(),
                 args.log_level,
             ),
         )
     } else if args.list {
         (
             "list content",
-            list_cpio_content(archive, &mut stdout, args.log_level),
+            list_cpio_content(
+                archive,
+                &mut stdout,
+                args.parts.as_ref(),
+                &args.patterns,
+                args.log_level,
+            ),
         )
     } else {
         unreachable!("no operation specified");
