@@ -557,8 +557,9 @@ impl Archive {
         &self,
         output_file: &mut W,
         source_date_epoch: Option<u32>,
+        mut size: u64,
         log_level: u32,
-    ) -> Result<()> {
+    ) -> Result<u64> {
         let mut next_ino = 0;
         let mut hardlink_ino = HashMap::new();
         let mut header;
@@ -575,17 +576,22 @@ impl Archive {
             if log_level >= LOG_LEVEL_DEBUG {
                 writeln!(std::io::stderr(), "{header:?}")?;
             };
-            header.write(output_file)?;
+            size += header.write(output_file)?;
             match &file.filetype {
                 Filetype::Hardlink { key, index: _ } => {
                     if header.filesize > 0 {
                         let hardlink = self.hardlinks.get(key).unwrap();
-                        copy_file_with_padding(&hardlink.location, hardlink.filesize, output_file)?;
+                        size += copy_file_with_padding(
+                            &hardlink.location,
+                            hardlink.filesize,
+                            output_file,
+                        )?;
                     }
                 }
                 Filetype::Symlink { target } => {
                     output_file.write_all(target.as_bytes())?;
-                    write_padding(output_file, header.filesize)?;
+                    size += u64::try_from(target.len()).unwrap();
+                    size += write_padding(output_file, header.filesize)?;
                 }
                 Filetype::EmptyFile
                 | Filetype::Directory
@@ -595,8 +601,8 @@ impl Archive {
                 | Filetype::Socket => {}
             }
         }
-        Header::trailer().write(output_file)?;
-        Ok(())
+        size += Header::trailer().write(output_file)?;
+        Ok(size)
     }
 }
 
@@ -662,12 +668,14 @@ impl Manifest {
         Ok(())
     }
 
+    // Return the size in bytes of the uncompressed data.
     pub fn write_archive(
         self,
         mut file: Option<std::fs::File>,
         source_date_epoch: Option<u32>,
         log_level: u32,
-    ) -> Result<()> {
+    ) -> Result<u64> {
+        let mut size = 0;
         if let Some(file) = file.as_ref() {
             self.apply_umask(file)?;
         }
@@ -675,16 +683,16 @@ impl Manifest {
             if archive.compression.is_uncompressed() {
                 if let Some(file) = file.as_mut() {
                     let mut writer = BufWriter::new(file);
-                    archive.write(&mut writer, source_date_epoch, log_level)?;
+                    size = archive.write(&mut writer, source_date_epoch, size, log_level)?;
                     writer.flush()?;
                 } else {
                     let mut stdout = std::io::stdout().lock();
-                    archive.write(&mut stdout, source_date_epoch, log_level)?;
+                    size = archive.write(&mut stdout, source_date_epoch, size, log_level)?;
                 }
             } else {
                 let mut compressor = archive.compression.compress(file, source_date_epoch)?;
                 let mut writer = BufWriter::new(compressor.stdin.as_ref().unwrap());
-                archive.write(&mut writer, source_date_epoch, log_level)?;
+                size = archive.write(&mut writer, source_date_epoch, size, log_level)?;
                 writer.flush()?;
                 drop(writer);
                 let exit_status = compressor.wait()?;
@@ -698,50 +706,52 @@ impl Manifest {
                 break;
             }
         }
-        Ok(())
+        Ok(size)
     }
 }
 
-fn copy_file_with_padding<W: Write>(path: &str, filesize: u32, writer: &mut W) -> Result<()> {
+fn copy_file_with_padding<W: Write>(path: &str, filesize: u32, writer: &mut W) -> Result<u64> {
     let file = std::fs::File::open(path).map_err(|e| e.add_prefix(path))?;
     let mut reader = std::io::BufReader::new(file);
-    let copied_bytes = std::io::copy(&mut reader, writer)?;
+    let mut copied_bytes = std::io::copy(&mut reader, writer)?;
     if copied_bytes != filesize.into() {
         return Err(Error::new(
             ErrorKind::UnexpectedEof,
             format!("Copied {copied_bytes} bytes from {path} but expected {filesize} bytes."),
         ));
     }
-    write_padding(writer, filesize)?;
-    Ok(())
+    copied_bytes += write_padding(writer, filesize)?;
+    Ok(copied_bytes)
 }
 
-pub fn write_padding<W: Write>(file: &mut W, written_bytes: u32) -> Result<()> {
+pub fn write_padding<W: Write>(file: &mut W, written_bytes: u32) -> Result<u64> {
     let padding_len = align_to_4_bytes(written_bytes);
     if padding_len == 0 {
-        return Ok(());
+        return Ok(0);
     }
     let padding = vec![0u8; padding_len.try_into().unwrap()];
-    file.write_all(&padding)
+    file.write_all(&padding)?;
+    Ok(padding_len.into())
 }
 
 #[cfg(test)]
 mod tests {
     use std::fs::{canonicalize, hard_link};
-    use std::path::PathBuf;
+    use std::io::Read;
 
     use super::*;
-    use crate::libc::tests::make_temp_dir;
+    use crate::temp_dir::TempDir;
+    use crate::tests::TEST_LOCK;
     use crate::LOG_LEVEL_WARNING;
 
-    pub fn make_temp_dir_with_hardlinks() -> Result<PathBuf> {
-        let temp_dir = make_temp_dir()?;
-        let path = temp_dir.join("a");
+    pub fn make_temp_dir_with_hardlinks() -> Result<TempDir> {
+        let temp_dir = TempDir::new()?;
+        let path = temp_dir.path.join("a");
         let mut file = std::fs::File::create(&path)?;
         file.set_permissions(PermissionsExt::from_mode(0o755))?;
         file.write_all(b"content")?;
-        hard_link(&path, temp_dir.join("b"))?;
-        hard_link(&path, temp_dir.join("c"))?;
+        hard_link(&path, temp_dir.path.join("b"))?;
+        hard_link(&path, temp_dir.path.join("c"))?;
         Ok(temp_dir)
     }
 
@@ -1015,7 +1025,7 @@ mod tests {
     #[test]
     fn test_file_from_line_location_hardlink() {
         let temp_dir = make_temp_dir_with_hardlinks().unwrap();
-        let path = temp_dir.join("a").to_str().unwrap().to_owned();
+        let path = temp_dir.path.join("a").to_str().unwrap().to_owned();
         let line = format!("{path}\ta\t\t644\t1\t2");
         let stat = symlink_metadata(&path).unwrap();
         let mtime = stat.mtime().try_into().unwrap();
@@ -1038,7 +1048,7 @@ mod tests {
 
         let line = format!(
             "{}/b\tb\t\t640\t3\t4\t1751413453",
-            temp_dir.to_str().unwrap()
+            temp_dir.path.to_str().unwrap()
         );
         let (file, umask) = File::from_line(line, &mut hardlinks).unwrap();
         assert_eq!(
@@ -1061,6 +1071,7 @@ mod tests {
 
     #[test]
     fn test_file_from_line_location_relative_directory() {
+        let _lock = TEST_LOCK.lock().unwrap();
         let line = "./tests\t\t\t510\t7\t42";
         let stat = symlink_metadata("tests").unwrap();
         let mtime = stat.mtime().try_into().unwrap();
@@ -1350,18 +1361,193 @@ mod tests {
         );
     }
 
+    #[test]
+    fn test_manifest_write_archive_empty_bzip2() {
+        let temp_dir = TempDir::new().unwrap();
+        let path = temp_dir.path.join("initrd.img");
+        let input = b"#cpio: bzip2 -3\n";
+        let manifest = Manifest::from_input(input.as_ref(), LOG_LEVEL_WARNING).unwrap();
+        let file = std::fs::File::create(&path).unwrap();
+        let size = manifest
+            .write_archive(Some(file), Some(1754439117), LOG_LEVEL_WARNING)
+            .unwrap();
+        assert_eq!(size, 124);
+        let mut written_file = std::fs::File::open(&path).unwrap();
+        let mut output = Vec::new();
+        let read = written_file.read_to_end(&mut output).unwrap();
+        assert_eq!(
+            output,
+            b"BZh31AY&SY\x12<\x9e\xb3\0\0\
+            \x0c^\0D\0(\0h\x802$\x14\0 \x001\
+            L\0\0\xd3(\x0d\x0fH\x88\x17A\xa8\x8eh!$\
+            \xe5l\xc6e\xf5\xba\xaf\x8b\xb9\"\x9c(H\x09\x1eOY\x80"
+        );
+        assert_eq!(read, 66);
+    }
+
+    #[test]
+    fn test_manifest_write_archive_empty_gzip() {
+        let temp_dir = TempDir::new().unwrap();
+        let path = temp_dir.path.join("initrd.img");
+        let input = b"#cpio: gzip -7\n";
+        let manifest = Manifest::from_input(input.as_ref(), LOG_LEVEL_WARNING).unwrap();
+        let file = std::fs::File::create(&path).unwrap();
+        let size = manifest
+            .write_archive(Some(file), Some(1754439117), LOG_LEVEL_WARNING)
+            .unwrap();
+        assert_eq!(size, 124);
+        let mut written_file = std::fs::File::open(&path).unwrap();
+        let mut output = Vec::new();
+        let read = written_file.read_to_end(&mut output).unwrap();
+        assert_eq!(
+            output,
+            b"\x1f\x8b\x08\0\0\0\0\0\0\x03307070\
+            4 \x0e\x10\xab\x0e\x1d8\xc1\x18!A\x8e\x9e>\xae\
+            A\x8a\x8a\x8a\x0c@\0\0N\xe5\x097|\0\0\0"
+        );
+        assert_eq!(read, 48);
+    }
+
+    #[test]
+    fn test_manifest_write_archive_empty_lz4() {
+        let temp_dir = TempDir::new().unwrap();
+        let path = temp_dir.path.join("initrd.img");
+        let input = b"#cpio: lz4 -4\n";
+        let manifest = Manifest::from_input(input.as_ref(), LOG_LEVEL_WARNING).unwrap();
+        let file = std::fs::File::create(&path).unwrap();
+        let size = manifest
+            .write_archive(Some(file), Some(1754439117), LOG_LEVEL_WARNING)
+            .unwrap();
+        assert_eq!(size, 124);
+        let mut written_file = std::fs::File::open(&path).unwrap();
+        let mut output = Vec::new();
+        let read = written_file.read_to_end(&mut output).unwrap();
+        assert_eq!(
+            output,
+            b"\x02!L\x18$\0\0\0\x7f0707010\
+            \x01\0\x13/10\x01\0#\x14B\x09\0\xe0TR\
+            AILER!!!\0\0\0\0"
+        );
+        assert_eq!(read, 44);
+    }
+
+    #[test]
+    fn test_manifest_write_archive_empty_lzma() {
+        let temp_dir = TempDir::new().unwrap();
+        let path = temp_dir.path.join("initrd.img");
+        let input = b"#cpio: lzma -1\n";
+        let manifest = Manifest::from_input(input.as_ref(), LOG_LEVEL_WARNING).unwrap();
+        let file = std::fs::File::create(&path).unwrap();
+        let size = manifest
+            .write_archive(Some(file), Some(1754439117), LOG_LEVEL_WARNING)
+            .unwrap();
+        assert_eq!(size, 124);
+        let mut written_file = std::fs::File::open(&path).unwrap();
+        let mut output = Vec::new();
+        let read = written_file.read_to_end(&mut output).unwrap();
+        assert_eq!(
+            output,
+            b"]\0\0\x10\0\xff\xff\xff\xff\xff\xff\xff\xff\0\x18\x0d\
+            \xdd\x04b3\x02;A\xe5P\x06\xe8\xc4\xa0\xd8\x89Z\
+            pL\xa1]\xb0mv\xe7&\xc4o\xff\xfe$\x90\0"
+        );
+        assert_eq!(read, 48);
+    }
+
+    #[test]
+    fn test_manifest_write_archive_empty_lzop() {
+        let temp_dir = TempDir::new().unwrap();
+        let path = temp_dir.path.join("initrd.img");
+        let input = b"#cpio: lzop -9\n";
+        let manifest = Manifest::from_input(input.as_ref(), LOG_LEVEL_WARNING).unwrap();
+        let file = std::fs::File::create(&path).unwrap();
+        let size = manifest
+            .write_archive(Some(file), Some(1754439117), LOG_LEVEL_WARNING)
+            .unwrap();
+        assert_eq!(size, 124);
+        let mut written_file = std::fs::File::open(&path).unwrap();
+        let mut output = Vec::new();
+        let read = written_file.read_to_end(&mut output).unwrap();
+        // The lzop magic is 9 bytes long. Then follows: 3x 16-bit version fields,
+        // 2x 8-bit method and level, 2x 32-bit flags and mode, and 64-bit mtime.
+        // Then follows the filename (8-bit size) and then the 32-bit CRC32 checksum.
+        output.splice(9..15, b"versio".to_owned());
+        output.splice(25..33, b"mtime-42".to_owned());
+        output.splice(34..38, b"CRCS".to_owned());
+        assert_eq!(
+            output,
+            b"\x89LZO\0\x0d\x0a\x1a\x0aversio\x03\
+            \x09\x03\0\0\x0d\0\0\0\0mtime-4\
+            2\0CRCS\0\0\0|\0\0\0%\xbc\x7f\
+            \x179\x1307F\x0010 \x05\x02\x0010 \
+            \x15\x01\0B\xe0\x01\x08TRAILER!!\
+            !\0@\0\x11\0\0\0\0\0\0"
+        );
+        assert_eq!(read, 91);
+    }
+
+    #[test]
+    fn test_manifest_write_archive_empty_xz() {
+        let temp_dir = TempDir::new().unwrap();
+        let path = temp_dir.path.join("initrd.img");
+        let input = b"#cpio: xz -6\n";
+        let manifest = Manifest::from_input(input.as_ref(), LOG_LEVEL_WARNING).unwrap();
+        let file = std::fs::File::create(&path).unwrap();
+        let size = manifest
+            .write_archive(Some(file), Some(1754439117), LOG_LEVEL_WARNING)
+            .unwrap();
+        assert_eq!(size, 124);
+        let mut written_file = std::fs::File::open(&path).unwrap();
+        let mut output = Vec::new();
+        let read = written_file.read_to_end(&mut output).unwrap();
+        assert_eq!(
+            output,
+            b"\xfd7zXZ\0\0\x01i\"\xde6\x02\0!\x01\
+            \x16\0\0\0t/\xe5\xa3\xe0\0{\0\x1c]\0\x18\
+            \x0d\xdd\x04c\x9d\x8a@Z1\xe4\xcb{\x1c\xc7\xc9\xc0\
+            \xef\x917N\x01]\xbd\xd5q\xc8\0\0N\xe5\x097\
+            \0\x014|\xcb{\x1f\xc2\x90B\x99\x0d\x01\0\0\0\0\x01YZ"
+        );
+        assert_eq!(read, 84);
+    }
+
+    #[test]
+    fn test_manifest_write_archive_empty_zstd() {
+        let temp_dir = TempDir::new().unwrap();
+        let path = temp_dir.path.join("initrd.img");
+        let input = b"#cpio: zstd -2\n";
+        let manifest = Manifest::from_input(input.as_ref(), LOG_LEVEL_WARNING).unwrap();
+        let file = std::fs::File::create(&path).unwrap();
+        let size = manifest
+            .write_archive(Some(file), Some(1754439117), LOG_LEVEL_WARNING)
+            .unwrap();
+        assert_eq!(size, 124);
+        let mut written_file = std::fs::File::open(&path).unwrap();
+        let mut output = Vec::new();
+        let read = written_file.read_to_end(&mut output).unwrap();
+        assert_eq!(
+            output,
+            b"(\xb5/\xfd\x04P\x15\x01\0\xc8070701\
+            010B0TRAILER!!!\0\
+            \0\0\0\x03\x10\0\x19\xde\x89?F\x95\xfb\x16m"
+        );
+        assert_eq!(read, 47);
+    }
+
     #[test]
     fn test_manifest_write_fail_compression() {
-        let temp_dir = make_temp_dir().unwrap();
+        let temp_dir = TempDir::new().unwrap();
         let root_dir = File::new(Filetype::Directory, ".", 0o755, 0x333, 0x42, 0x6841897B);
         let archive = Archive::with_files_compressed(vec![root_dir], Compression::Failing);
         let manifest = Manifest::new(vec![archive], 0o022);
-        let file = std::fs::File::create(temp_dir.join("initrd.img")).unwrap();
+        let file = std::fs::File::create(temp_dir.path.join("initrd.img")).unwrap();
         let got = manifest
             .write_archive(Some(file), None, LOG_LEVEL_WARNING)
             .unwrap_err();
-        assert_eq!(got.to_string(), "false failed: exit status: 1");
-        assert_eq!(got.kind(), ErrorKind::Other);
+        assert!(
+            matches!(got.kind(), ErrorKind::Other if got.to_string() == "false failed: exit status: 1")
+                || matches!(got.kind(), ErrorKind::BrokenPipe)
+        );
     }
 
     #[test]
@@ -1405,8 +1591,8 @@ mod tests {
             File::new(Filetype::EmptyFile, "fstab", 0x644, 0x339, 0x48, 0x6E44C280),
         ]);
         let mut output = Vec::new();
-        archive
-            .write(&mut output, Some(0x6B49D200), LOG_LEVEL_WARNING)
+        let size = archive
+            .write(&mut output, Some(0x6B49D200), 0, LOG_LEVEL_WARNING)
             .unwrap();
         assert_eq!(
             std::str::from_utf8(&output).unwrap(),
@@ -1435,12 +1621,13 @@ mod tests {
             00000000000000000000000000000000000000000000000B00000000\
             TRAILER!!!\0\0\0\0",
         );
+        assert_eq!(size, 952);
     }
 
     #[test]
     fn test_archive_write_hardlinks() {
         let temp_dir = make_temp_dir_with_hardlinks().unwrap();
-        let path = temp_dir.join("a").to_str().unwrap().to_owned();
+        let path = temp_dir.path.join("a").to_str().unwrap().to_owned();
         // This archive data is the output of test_file_from_line_location_hardlink.
         let archive = Archive::with_files_and_hardlinks(
             vec![
@@ -1470,7 +1657,9 @@ mod tests {
             HashMap::from([(8921120, Hardlink::with_references(&path, 7, 2))]),
         );
         let mut output = Vec::new();
-        archive.write(&mut output, None, LOG_LEVEL_WARNING).unwrap();
+        let size = archive
+            .write(&mut output, None, 0, LOG_LEVEL_WARNING)
+            .unwrap();
         assert_eq!(
             std::str::from_utf8(&output).unwrap(),
             "07070100000000000081A40000000100000002000000026861C7C5\
@@ -1483,5 +1672,6 @@ mod tests {
             00000000000000000000000000000000000000000000000B00000000\
             TRAILER!!!\0\0\0\0",
         );
+        assert_eq!(size, 356);
     }
 }
